/*
  This file is part of quick-memo
  Copyright (C) 2014 Stefano Verzegnassi

    This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License 3 as published by
  the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
  along with this program. If not, see http://www.gnu.org/licenses/.
*/

import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Components.Themes.Ambiance 0.1

Column {
    id: rootItem

    width: parent.width
    spacing: units.gu(2)

    property alias model: repeater.model
    property alias delegate: repeater.delegate

    property alias header: headerLoader.sourceComponent
    property alias footer: footerLoader.sourceComponent

    property bool altStyle: false

    signal listChanged()

    Loader {
        id: headerLoader
        width: parent.width
    }

    Repeater {
        id: repeater
        width: parent.width

        model: ListModel {
            id: dataModel

            ListElement {
                checked: false
                text: ""
            }
        }

        Component.onCompleted: {
            if (rootItem.altStyle)
                repeater.delegate = altStyleDelegate
            else
                repeater.delegate = delegate
        }
    }


    // TODO: When user adds an element in the list, the new element should gain focus.
    Loader {
        id: footerLoader
        width: parent.width

        sourceComponent: altStyle ? altStyleFooter : footer

        Component {
            id: altStyleFooter

            AbstractButton {
                width: parent.width
                height: Math.max(addListBtnImage.height, addListBtnLabel.height)
                opacity: enabled ? 1.0 : 0.5

                enabled: {
                    if (rootItem.model.count > 0) {
                        if (rootItem.model.get(rootItem.model.count - 1).text !== "") {
                            return true
                        } else {
                            return false
                        }
                    } else {
                        return true
                    }
                }

                Row {
                    spacing: units.gu(2)
                    anchors.fill: parent

                    Item {
                        id: addListBtnImage
                        implicitWidth: units.gu(4.25)
                        implicitHeight: units.gu(4)

                        Icon {
                            anchors { fill: parent; margins: units.gu(0.5) }
                            name: "add"
                        }
                    }

                    Label {
                        id: addListBtnLabel
                        text: i18n.tr("Add element")
                        anchors.verticalCenter: addListBtnImage.verticalCenter
                    }
                }

                onClicked: {
                    rootItem.model.append({"checked": false, "text": ""})
                    // Here we don't send listChanged signal, since it is an empty item that should not be saved
                }
            }
        }


        Component {
            id: footer

            Item {
                width: footerBtn.width
                height: footerBtn.height + units.gu(2)

                Button {
                    id: footerBtn
                    anchors.bottom: parent.bottom
                    text: i18n.tr("Add element")
                    color: UbuntuColors.orange

                    enabled: {
                        if (rootItem.model.count > 0) {
                            if (rootItem.model.get(rootItem.model.count - 1).text !== "") {
                                return true
                            } else {
                                return false
                            }
                        } else {
                            return true
                        }
                    }

                    onClicked: {
                        rootItem.model.append({"checked": false, "text": ""})
                        // Here we don't send listChanged signal, since it is an empty item that should not be saved
                    }
                }
            }
        }
    }

    function exportModel() {
        var list = []

        if (rootItem.model.count > 0 && rootItem.model.get(0).text !== "") {
            for (var i=0; i<rootItem.model.count; i++) {
                if (i != rootItem.model.count && rootItem.model.get(i).text != "") {
                    list[i] = rootItem.model.get(i)
                }
            }
        }

        return list
    }


    // *** DELEGATES ***

    // TODO: TextArea should lose their focus when user touches a different part of screen. InverseMouseArea seems to be a solution.
    Component {
        id: delegate

        ListItem.Empty {
            width: parent ? parent.width : undefined
            height: textArea.height

            showDivider: false

            // ListItem is removable for all items, except the first when the text field is empty.
            removable: rootItem.model.count > 1 || (rootItem.model.get(0).text !== "")
            onItemRemoved: {
                // Send the signal before removing the item. This avoids a ReferenceError.
                rootItem.listChanged()
                rootItem.model.remove(model.index)
            }

            Row {
                id: layoutDelegate
                spacing: units.gu(2)
                anchors.fill: parent

                CheckBox {
                    id: checkBox
                    checked: model.checked
                    onCheckedChanged: layoutDelegate.updateModel()
                }

                TextField {
                    id: textArea
                    width: parent.width - checkBox.width - parent.spacing
                    placeholderText: i18n.tr("Add text")
                    onTextChanged: layoutDelegate.updateModel()

                    hasClearButton: false

                    Component.onCompleted: {
                        if (model.text) {
                            text = model.text
                        }
                    }
                }

                function updateModel() {
                    rootItem.model.set(model.index, {"checked": checkBox.checked, "text": textArea.text})
                    rootItem.listChanged()
                }
            }
        }
    }

    Component {
        id: altStyleDelegate
        Row {
            id: layoutDelegate
            spacing: units.gu(1)
            width: parent ? parent.width : undefined
            height: Math.max(checkBox.height, textArea.height, deleteBtn.height)

            CheckBox {
                id: checkBox

                checked: model.checked
                onCheckedChanged: layoutDelegate.updateModel()

                style: Item {
                    implicitWidth: units.gu(4.25)
                    implicitHeight: units.gu(4)

                    Image {
                        anchors { fill: parent; margins: units.gu(0.5) }

                        source: checkBox.checked ? "../img/select.svg" : "../img/unselect.svg"
                    }
                }
            }

            TextField {
                id: textArea
                width: parent.width - (checkBox.width + (parent.spacing * 2) + deleteBtn.width)
                font.strikeout: checkBox.checked
                text: model.text
                onTextChanged: layoutDelegate.updateModel()

                hasClearButton: false

                style: TextFieldStyle {
                    frameSpacing: 0
                    overlaySpacing: units.gu(0.5)

                    background: Item {
                        id: bgItem
                        anchors.fill: parent

                        /*InverseMouseArea {
                            anchors.fill: parent
                            onClicked: rootItem.focus = false
                        }*/

                        Rectangle {
                            width: bgItem.width
                            height: units.gu(0.1)
                            y: textArea.height - height - units.gu(0.5)
                            color: Theme.palette.selected.backgroundText
                            visible: textArea.highlighted
                        }
                    }
                }
            }

            IconButton {
                id: deleteBtn
                source: "close"
                onClicked: {
                    // Send the signal before removing the item. This avoids a ReferenceError.
                    rootItem.listChanged()
                    rootItem.model.remove(model.index)
                }

                // Delete button is visible for all items, except the first when the text field is empty.
                // Also, make the button visible only when textArea is focused.
                visible: textArea.focus && (rootItem.model.count > 1 || (rootItem.model.get(0).text !== ""))
                enabled: visible
           }

            function updateModel() {
                rootItem.model.set(model.index, {"checked": checkBox.checked, "text": textArea.text})
                rootItem.listChanged()
            }
        }
    }
}
