/*
  This file is part of quick-memo
  Copyright (C) 2014 Stefano Verzegnassi

    This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License 3 as published by
  the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
  along with this program. If not, see http://www.gnu.org/licenses/.
*/

import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Components.Popups 1.0

PageWithBottomEdge {
    id: mainPage
    title: i18n.tr("Overview")
    flickable: useGridView ? null : viewLoader.item

    // TODO: Switch to PageHeads. Do it also for MultiSelectionHandler

    Action {
        id: openAbout
        text: i18n.tr("About...")
        iconName: "help"
        onTriggered: pageStack.push(Qt.resolvedUrl("AboutPage.qml"))
    }

    Action {
        id: addNote
        text: i18n.tr("Add note")
        iconName: "note-new"
        onTriggered: pageStack.push(Qt.resolvedUrl("AddMemo.qml"))
    }

    Action {
        id: switchView
        text: useGridView ? i18n.tr("Switch to one-column list") : i18n.tr("Switch to grid")
        iconName: useGridView ? "view-list-symbolic" : "view-grid-symbolic"
        onTriggered: useGridView = !useGridView
    }

    tools: ToolbarItems {
        ToolbarButton { action: openAbout }
        ToolbarButton { action: addNote }
        ToolbarButton { action: switchView }
    }

    // Used for switching from HorizontalFlowListView to ListView, and vice versa.
    Loader {
        id: viewLoader
        anchors.fill: parent

        sourceComponent: useGridView ? horizontalFlowListView : listView
    }

    /* A sort of GridView that scrolls horizontally, wrapping delegates to create rows or columns of items.
       Items are positioned next to each other from top to bottom
       until the height of the Flow is exceeded, then wrapped to the next column.
       http://qt-project.org/doc/qt-5/qml-qtquick-flow.html */
    Component {
        id: horizontalFlowListView

        HorizontalFlowListView {
            id: memoView
            anchors { fill: parent; topMargin: units.gu(2) }
            clip: false

            spacing: units.gu(1)

            model: notes.model
            delegate: Delegate {
                id: viewDelegate
                width: (memoView.width * 0.5) - (memoView.spacing * 3)

                onClicked: {
                    if (multiSelection.active) {
                        selected = multiSelection.selectUnselectItem(model.index)
                    } else {
                        /* Workaround: clear the pageStack before pushing memoPage.
                            We do this because when we remove an item from memoPage, the whole
                            application freezes. We take also another advantage: less memory usage
                            (no delagates images to keep loaded). */
                        /* UPDATE: That was true with the earlier code. After switching to SortFilterModel,
                           the issue does not live anymore. We save anyway memory, and that's good. */
                        pageStack.clear()
                        pageStack.push(memoPage, {index: model.index})
                    }
                }

                onPressAndHold: {
                    if (!multiSelection.active) {
                        multiSelection.active = true
                        selected = multiSelection.selectUnselectItem(model.index)
                    }
                }
                Connections {
                    target: multiSelection
                    onActiveChanged: viewDelegate.selected = false
                }
            }
        }
    }

    // The classic and boring ListView...
    Component {
        id: listView

        ListView {
            id: memoView
            anchors { fill: parent; margins: units.gu(2) }
            clip: false

            spacing: units.gu(1)

            model: notes.model
            delegate: ListViewDelegate {
                id: viewDelegate
                width: memoView.width

                onClicked: {
                    if (multiSelection.active) {
                        selected = multiSelection.selectUnselectItem(model.index)
                    } else {
                        /* Workaround: clear the pageStack before pushing memoPage.
                            We do this because when we remove an item from memoPage, the whole
                            application freezes. We take also another advantage: less memory usage
                            (no delagates images to keep loaded). */
                        /* UPDATE: That was true with the earlier code. After switching to SortFilterModel,
                           the issue does not live anymore. We save anyway memory, and that's good. */
                        pageStack.clear()
                        pageStack.push(memoPage, {index: model.index})
                    }
                }

                onPressAndHold: {
                    if (!multiSelection.active) {
                        multiSelection.active = true
                        selected = multiSelection.selectUnselectItem(model.index)
                    }
                }
                Connections {
                    target: multiSelection
                    onActiveChanged: viewDelegate.selected = false
                }
            }
        }
    }

    // Used to manage multi-selection in ListViews
    MultiSelectionHandler {
        id: multiSelection

        target: mainPage
        title: (multiSelection.count == 1) ? i18n.tr("%1 item selected").arg(multiSelection.count)
                                           : i18n.tr("%1 items selected").arg(multiSelection.count)
        tools: ToolbarItems {
            back: ToolbarButton {
                action: Action {
                    text: i18n.tr("Cancel")
                    iconName: "back"
                    onTriggered: multiSelection.active = false
                }
            }

            ToolbarButton {
                id: colorToolBtn
                action: Action {
                    text: i18n.tr("Change note color")
                    iconSource: "../img/palette.svg"
                    onTriggered: PopupUtils.open(colorNotePopover, colorToolBtn)
                }
            }

            ToolbarButton {
                action: Action {
                    id: deleteMemo
                    text: i18n.tr("Delete note")
                    iconName: "delete"
                    onTriggered: {
                        PopupUtils.open(deleteDialog)
                    }
                }
            }
        }

        onIndexAdded: console.log("Added index:", index)
        onIndexRemoved: console.log("Index removed:", index)

        Component {
            id: colorNotePopover

            ColorDialog {
                id: colorDialog
                showTick: false
                // A color not listed in 'colors' property, just to avoid that oldColor and selectedColor result the same.
                // TODO: If there's just a single element selected, we can import the right color of the memo.
                selectedColor: "#ababab"

                onColorPicked: {
                    if (isChanged) {
                        for (var i=0; i<multiSelection.indexes.length; i++) {
                            notes.setNoteProperty(multiSelection.indexes[i], "color", selectedColor)
                        }
                    } else {
                        console.log("Color not changed")
                    }

                    multiSelection.active = false
                    PopupUtils.close(colorDialog)
                }
            }
        }

        Component {
            id: deleteDialog
            Dialog {
                id: deleteDialogue
                title: i18n.tr("Delete note(s)")
                text: i18n.tr("Are you sure?")

                Button {
                    text: i18n.tr("Cancel")
                    gradient: UbuntuColors.greyGradient
                    onClicked: PopupUtils.close(deleteDialogue)
                }
                Button {
                    text: i18n.tr("Delete")
                    color: UbuntuColors.orange
                    onClicked: {
                        notes.deleteNotes(multiSelection.indexes)
                        multiSelection.active = false
                        PopupUtils.close(deleteDialogue)
                    }
                }
            }
        }
    }

    bottomEdgeTitle: i18n.tr("New memo")
    bottomEdgePageComponent: AddMemo {}
}
