/*
  This file is part of quick-memo
  Copyright (C) 2014 Stefano Verzegnassi

    This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License 3 as published by
  the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
  along with this program. If not, see http://www.gnu.org/licenses/.
*/

import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.Popups 1.0
import "dateHelper.js" as DateHelper

import Ubuntu.Components.Themes.Ambiance 0.1

Page {
    id: memoPage
    property int index

    title: i18n.tr("Details")

    Action {
        id: deleteMemo
        text: i18n.tr("Delete note")
        iconName: "delete"
        onTriggered: PopupUtils.open(deleteDialog)
    }
    Component {
        id: deleteDialog
        Dialog {
            id: deleteDialogue
            title: i18n.tr("Delete note")
            text: i18n.tr("Are you sure?")

            Button {
                text: i18n.tr("Cancel")
                gradient: UbuntuColors.greyGradient
                onClicked: PopupUtils.close(deleteDialogue)
            }
            Button {
                text: i18n.tr("Delete")
                color: UbuntuColors.orange
                onClicked: {
                    PopupUtils.close(deleteDialogue)
                    pageStack.pop();
                    pageStack.push(mainPage)
                    notes.deleteNote(index)
                }
            }
        }
    }

    Action {
        id: addImage
        text: i18n.tr("Add image")
        iconName: "insert-image"
        onTriggered: picsView.importImageFromContentHub()
    }

    Action {
        id: colorToolAction
        text: i18n.tr("Change note color")
        iconSource: "../img/palette.svg"
        onTriggered: PopupUtils.open(colorNotePopover, colorToolBtn)
    }
    Component {
        id: colorNotePopover

        ColorDialog {
            id: colorDialog

            selectedColor: bg.color

            onColorPicked: {
                if (isChanged) {
                    bg.color = selectedColor
                }
                PopupUtils.close(colorDialog)
            }
        }
    }

    tools: ToolbarItems {
        back: ToolbarButton {
            action: Action {
                iconName: "back"
                onTriggered: {
                    // Check if a saveNote request was made. If so, stop the timer and save before exiting the page.
                    if (autoSaveTimer.running) {
                        console.log("It's ok. Saving the note before closing the page...")
                        autoSaveTimer.running = false
                        saveNote()
                    }

                    pageStack.pop()

                    /* Workaround: see MainPage.qml, comment for mainView.delegate.onClicked().
                        We simply re-push in the stack what we removed earlier.*/
                    pageStack.push(mainPage)
                }
            }
        }

        ToolbarButton { id: colorToolBtn; action: colorToolAction }
        ToolbarButton { action: addImage }
        ToolbarButton { action: deleteMemo }
    }

    // Background
    Rectangle {
        id: bg
        anchors.fill: parent
        z: -10; opacity: 0.5
        color: notes.model.get(index).contents.color

        onColorChanged: updateNote()
    }

    /* MainView clips its content, so that it does not overlap the header.
       We need to bypass this, using another Rectangle that specifically overlap the header.
    */
    Rectangle {
        id: headerBg
        parent: root.header
        z: -10; opacity: 0.5

        width: root.width
        height: root.header.height
        color: bg.color
    }

    Flickable {
        id: flickable
        anchors.fill: parent

        // FIXME: Strange behaviour on contentHeight
        contentHeight: content.height + units.gu(4)
        interactive: true

        Column {
            id: content
            anchors { top: parent.top; left: parent.left; right: parent.right; margins: units.gu(2) }
            spacing: units.gu(4)

            Column {
                width: parent.width
                spacing: units.gu(2)

                NoteTextArea {
                    id: title
                    font.weight: Font.Bold
                    font.pixelSize: FontUtils.sizeToPixels("large")

                    Component.onCompleted: {
                        if (notes.model.get(index).contents.title == "")
                            placeholderText = i18n.tr("No title")
                        else
                            text = notes.model.get(index).contents.title
                    }

                    onTextChanged: {
                        updateNote()
                    }
                }

                NoteTextArea {
                    id: desc

                    Component.onCompleted: {
                        if (notes.model.get(index).contents.text == "")
                            placeholderText = i18n.tr("No description")
                        else
                            text = notes.model.get(index).contents.text
                    }

                    onTextChanged: {
                        updateNote()
                    }
                }
            }

            Column {
                width: parent.width
                spacing: units.gu(1)

                Label {
                    text: i18n.tr("List:")
                    font.weight: Font.DemiBold
                }

                ListManager {
                    id: listManager
                    width: parent.width

                    // Load listManager with an alternative style
                    altStyle: true

                    // Append list items provided by NoteModel
                    Component.onCompleted: {
                        listManager.model.clear()
                        for (var i=0; i<notes.model.get(index).contents.list.length; i++) {
                            listManager.model.append(JSON.parse(JSON.stringify(notes.model.get(index).contents.list[i])))
                        }
                    }

                    onListChanged: updateNote()
                }
            }


            Column {
                width: parent.width
                spacing: units.gu(1)

                Label {
                    text: i18n.tr("Related pictures:")
                    font.weight: Font.DemiBold
                }

                PicturesManager {
                    id: picsView
                    spacing: units.gu(2)

                    delegateHeight: units.gu(12)
                    delegateWidth: delegateHeight

                    // Append pictures items provided by NoteModel
                    Component.onCompleted: {
                        picsView.model.clear()
                        for (var i=0; i<notes.model.get(index).contents.pictures.length; i++) {
                            picsView.model.append(JSON.parse(JSON.stringify(notes.model.get(index).contents.pictures[i])))
                        }
                    }

                    onPicsModelChanged: updateNote()
                }
            }

            Label {
                id: updateLabel
                fontSize: "x-small"
                //font.italic: true
                x: parent.width - width

                Component.onCompleted: refresh()
                function refresh() {
                    updateLabel.text = i18n.tr("Last update: %1").arg(Qt.formatDateTime(DateHelper.parseDate(notes.model.get(index).contents.date), "d MMM yyyy, hh:mm:ss"))
                }
            }
        }
    }

    function saveNote() {
        var obj = JSON.parse(JSON.stringify(notes.model.get(index)))

        obj.contents.title = title.text
        obj.contents.text = desc.text
        // The char '#' from the hex color is not correctly parse by JSON. We add an empty string, so that it works well.
        obj.contents.color = "" +  bg.color
        obj.contents.list = listManager.exportModel()
        obj.contents.pictures = picsView.exportModel()

        notes.editNote(memoPage.index, obj)
        updateLabel.refresh()
    }

    // Used for auto saving while editing fields
    Timer {
        id: autoSaveTimer
        interval: 1000   // Is this a good timing?

        /* We use 'onTextChanged' (and similars) signals to know when is required to update the note.
          However, when this page is pushed, and the content is shown for the first time, those signals are
            emitted. We need to stop the timer when that happens.*/
        property bool initialized: false
        onTriggered: {
            if (!autoSaveTimer.initialized) {
                autoSaveTimer.initialized = true
                console.log("Note loaded!")
            } else {
                console.log("autoSaveTimer triggered... updating the note!")
                saveNote()
            }
        }
    }

    function updateNote() {
        autoSaveTimer.restart()
    }
}
